/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
 *  Author: Sibylle Kohler
 */

var deviceArray=[];

Ext.define('DSS.component.myTextedit', {
    extend : 'Ext.form.field.Text',
    allowBlank : false,
    valueToRaw : function (value)
    {
        if (value==null)
            return "";
        return unescape(value);
    },
    rawToValue : function (value)
    {
        if (value =="")
            return null;
        return escape(value);
    },
    constructor : function(config) {
    	config.fieldLabel=_('Message');
    	config.labelWidth=100;
    	config.padding='0 0 0 10';
    	config.style={
    		color: '#005138',
        	'font-family': 'tahoma,arial,verdana,sans-serif'
    	};
    	config.width=700;
    	this.callParent(arguments);
    }
    
});


Ext.define('DSS.component.MobileSelector', {
    extend : 'DSS.component.Base',
    checkboxArray:[],
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
        return 150;
    },
    updateCheckbox :function() {
    	this.checkboxArray=[];
    	this.basePanel.removeAll();
    	if (deviceArray==null)
    		return;
    	if (deviceArray.length==null)
    		return;
    	for (var iIndex=0;iIndex<deviceArray.length;iIndex++) {
    		var oCheckbox=Ext.create('Ext.form.field.Checkbox',{
    			id:deviceArray[iIndex].uuid,
    			boxLabel:deviceArray[iIndex].name + ": " + deviceArray[iIndex].appName,
    			style:{
    	    		color: '#005138',
    	        	'font-family': 'tahoma,arial,verdana,sans-serif'
    	    	}
    		});
    		this.checkboxArray.push(oCheckbox);
    		this.basePanel.add(oCheckbox);
    	}
    	if ((deviceArray.length)>8)
    		this.basePanel.setHeight(240);
    },
    basePanel:null,
    rbBox:null,
    constructor : function(config) {
       var me = this;       
       this.basePanel=Ext.create('Ext.form.FieldSet',{
    	   padding: '5 0 0 10',
    	   margin: '0 70 0 126',
    	   title:_('choose devices'),
    	   autoScroll:true
       });
       this.rbBox=Ext.create('Ext.form.RadioGroup',{
    	fieldLabel:_('target devices:'),
       	columns:1,
       	vertical: true,
       	style: {
    		color: '#005138',
        	'font-family': 'tahoma,arial,verdana,sans-serif'
    	},
       	items: [
                   { boxLabel: _('all devices'), name: 'uuid_rb', inputValue: '1' },
                   { boxLabel: _('specific devices'), name: 'uuid_rb', inputValue: '2'}
           ],
       listeners: {
    	   'change': function () {
    		   if (this.getValue().uuid_rb==='2')
    			   me.basePanel.setDisabled(false);
    		   else
    			   me.basePanel.setDisabled(true);
    			   
    	   }
       }
       });
       this.checkboxArray=[];
        Ext.apply(this, {
     	    padding: '0 0 0 10',
        	layout: 'auto',
            items : [
                this.rbBox,
                this.basePanel
            ]
        });         
        this.rbBox.setValue({uuid_rb:'1'});
        this.callParent(arguments);
    },
    reset: function (){
    	for (var iIndex=0;iIndex<this.checkboxArray.length;iIndex++)
    		this.checkboxArray[iIndex].setValue(false);
    },
    setValue:function (value){
    	this.reset();
    	var fMulti=false;
    	for (var jIndex=0;jIndex<value.length;jIndex++) {
    		if (value[jIndex]=='*')
    			fMulti=true;
    		else
    			for (var iIndex=0;iIndex<this.checkboxArray.length;iIndex++)
    				if (this.checkboxArray[iIndex].getId()==value[jIndex])
    					this.checkboxArray[iIndex].setValue(true);
    	}
    	if (fMulti) {
    		this.rbBox.setValue({uuid_rb:'1'});
    	} else {
    		this.rbBox.setValue({uuid_rb:'2'});
    	}
    },
    isValid:function(){
    	for (var iIndex=0;iIndex<this.checkboxArray.length;iIndex++)
    		if (this.checkboxArray[iIndex].getValue())
    			return true;
    	if (this.rbBox.getValue().uuid_rb=='1')
    		return true;
        return false;
    },
    getValue : function(){
    	var oArray=[];
    	if (this.rbBox.getValue().uuid_rb==='2') {
	    	for (var iIndex=0;iIndex<this.checkboxArray.length;iIndex++)
	    		if (this.checkboxArray[iIndex].getValue())
	    			oArray.push(this.checkboxArray[iIndex].getId());
    	} else {
    		oArray.push('*');
    	}
    	return oArray;
    }
});


Ext.define('DSS.component.targetColumn', {
    extend : 'Ext.grid.column.Column',
    alias : 'widget.targetcolumn',
    renderer: function(val, metadata, record, rowIdx, colIdx, store, view) {
    	var sString="<table>";
    	var fMulti=false;
    	var bValid = false;
    	for (var iIndex=0;iIndex<val.length;iIndex++) {
    		if (val[iIndex]=='*') {
    			fMulti=true;
    		} else {
    			for (var jIndex=0;jIndex<deviceArray.length;jIndex++) {
    				if (deviceArray[jIndex].uuid==val[iIndex]) {
    					sString +="<tr><td>" +deviceArray[jIndex].name +": " + deviceArray[jIndex].appName + "</td></tr>";
    					bValid = true;
    				}
    			}
    			if (!bValid) {
    			    sString +='<tr><td style="color:#ff0000;">' +_('Device not connected')+ '</td></tr>';
    			}
    		}
    	}
    	if (!fMulti)
    		return sString +="</table>";
    	else
    		return _("all registered devices");
    }
});

Ext.define('DSS.addon.EventPushNotification', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : _('Event-Push-Notification'),
        appId : 'event-push-notification',
        appVersion : '1.0.7',
        dSSVersion : '1.9.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },

    getContent : function() {
        var me=this;
        this.mobileSelector=Ext.create('DSS.component.MobileSelector', {});
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                storeConfig: {
                    storeId : 'pushStore',
                    fields : [ {name:'disabled', sortType:'asBoolean'}, 
                               {name:'name', sortType:'asUCText'}, 
                               {name:'uuid', sortType:'asUCText'}, 
                               {name:'id'}, 
                               {name:'message', sortType:'asUCText'}, 
                               {name:'triggers', sortType:'asTrigger'}, 
                               {name:'conditions',  sortType: 'asUCText' },
                               {name:'grouping',  sortType: 'asUCText' },
                               {name:'defaultFooter', sortType:'asBoolean'} ]
                },
                multiselect:true,
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                }, {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn', 
                    width : 200
                }, {
                    header : _('Trigger'),
                    dataIndex : 'triggers', 
                    maintainFlex : true,
                    flex : 1,
                    xtype: 'triggercolumn'
                }, {
                    header : _('Target'),
                    dataIndex : 'uuid', 
                    maintainFlex : true,
                    flex : 1,
                    xtype: 'targetcolumn'
                },{
                    header : _('Message'),
                    dataIndex : 'message',
                    xtype:'textcolumn',
                    flex : 1
                },{
                    header: _('Conditions'),
                    dataIndex: 'conditions', // alibi index
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 1,
                    customRendererOptions:{hideEnabled:true,hideWeekdays:false,hideGenericSystemStates:false,hideZoneStates:false,hideTimeframe:false,filterSystemState:[],
                    	stateTitles:[{name:'daylight',value:true,title:_('grid_daylight_true')},{name:'daylight',value:false,title:_('grid_daylight_false')},
                    	             {name:'daynight',value:true,title:_('grid_daynight_true')},{name:'daynight',value:false,title:_('grid_daynight_false')},
                    	             {name:'panic',value:'1',title:_('grid_panic_active')},{name:'panic',value:'2',title:_('grid_panic_inactive')},
                    	             {name:'presence',value:'present',title:_('grid_presence_present')},{name:'presence',value:'absent',title:_('grid_presence_absent')},
                    	             {name:'fire',value:'1',title:_('grid_fire_active')},{name:'fire',value:'2',title:_('grid_fire_inactive')},
                    	             {name:'hail',value:'1',title:_('grid_hail_active')},{name:'hail',value:'2',title:_('grid_hail_inactive')},
                    	             {name:'wind',value:'1',title:_('grid_wind_active')},{name:'wind',value:'2',title:_('grid_wind_inactive')},
                    	             {name:'rain',value:'1',title:_('grid_rain_active')},{name:'rain',value:'2',title:_('grid_rain_inactive')},
                    	             {name:'alarm',value:'1',title:_('grid_alarm_active')},{name:'alarm',value:'2',title:_('grid_alarm_inactive')},
                    	             {name:'alarm2',value:'1',title:_('grid_alarm2_active')},{name:'alarm2',value:'2',title:_('grid_alarm2_inactive')},
                    	             {name:'alarm3',value:'1',title:_('grid_alarm3_active')},{name:'alarm3',value:'2',title:_('grid_alarm3_inactive')},
                    	             {name:'alarm4',value:'1',title:_('grid_alarm4_active')},{name:'alarm4',value:'2',title:_('grid_alarm4_inactive')},
                    	             {name:'holiday', value:'on',title:_('grid_holiday_on')},{name:'holiday', value:'off',title:_('grid_holiday_off')}]}
                }],     
                buttonText: {
                    newButton: _('New message'),
                    editButton: _('Edit message'),
                    enableButton:_('Activate message'),
                    disableButton:_('Deactivate message'),
                    testButton: _('Test message'),
                    deleteButton: _('Delete message')
                },

                editorConfig:{
                    title : _('Messageproperties'),
                    staticControls : [{
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type : 'hidden',
                        label : _('id'),
                        id : 'id'
                    }],
                    tabControls :  [{
                        tabLabel : _('Trigger'),
                        tabDescription : _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                        content : [ {
                            type : 'TriggerConfigurator',
                            id : 'triggers',
                            flex : true
                        } ]
                    } , {
                        tabLabel : _('Message'),
                        tabDescription : _('Provide the Message, which will be send.'),
                        content : [ {
                            type : 'custom',
                            id : 'message',
                            label : _('Message'),
                            control:Ext.create('DSS.component.myTextedit',{})
                        },{
                            type : 'custom',
                            label : _('Target'),
                            id : 'uuid',
                            control : this.mobileSelector,
                            flex : true
                        }]
                    } , {
                        tabLabel : _('Conditions'),
                        tabDescription : _('Choose the conditions.'),
                        content : [{
                            type : 'ConditionConfigurator',
                            id : 'conditions',
                            conditions:[
                                        {type:'singleItem',title:_('state_daynight'),entry:[
                                              {type:'state',name:'daylight',value:true,title:Ext.String.format2(_('daynight_bright'),{
		                                            	duskTime:dss.staticDataModel.getDusk(),
		                                            	dawnTime:dss.staticDataModel.getDawn()
		                                            }),gridText:Ext.String.format2(_('grid_daynight_twilight_true'),{
		                                            	duskTime:dss.staticDataModel.getDusk(),
		                                            	dawnTime:dss.staticDataModel.getDawn()
		                                            })},
                                              {type:'state',name:'daylight',value:false,title:Ext.String.format2(_('daynight_dark'),{
	                                            	duskTime:dss.staticDataModel.getDusk(),
	                                            	dawnTime:dss.staticDataModel.getDawn()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_twilight_false'),{
	                                            	duskTime:dss.staticDataModel.getDusk(),
	                                            	dawnTime:dss.staticDataModel.getDawn()
	                                            })},
                                              {type:'state',name:'daynight',value:true,title:Ext.String.format2(_('daynight_day'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_daynight_true'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            })},
                                              {type:'state',name:'daynight',value:false,title:Ext.String.format2(_('daynight_night'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_daynight_false'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            })}
	                                            
	                                            ]},
                                        {type:'timeframe',title:_('Timeframe')},
                                        {type:'weekdays',title:_('Weekdays')},
                                        {type:'deviceState',title:_('automatic devices')},
                                        {type:'singleItem',title:_('state_presence'),entry: [
                                               {type:'state',name:'presence',value:'present',title:_('presence_present'),gridText:_('grid_presence_present')},
                                               {type:'state',name:'presence',value:'absent',title:_('presence_absent'),gridText:_('grid_presence_absent')}]},                                               
                                        {type:'singleItem',title:_('state_holiday'),entry: [
                                               {type:'state',name:'holiday',value:'on',title:_('holiday_on'),gridText:_('grid_holiday_on')},
                                               {type:'state',name:'holiday',value:'off',title:_('holiday_off'),gridText:_('grid_holiday_off')}]},
	                                   {type:'multiItem',title:_('weather trigger'),entry: [
	                                            {type:'state',name:'rain',value:'1',title:_('rain active'),gridText:_('grid_rain_active'),group:1},
	                                            {type:'state',name:'rain',value:'2',title:_('rain inactive'),gridText:_('grid_rain_inactive'),group:1},
	                                            {type:'state',name:'wind',value:'1',title:_('wind active'),gridText:_('grid_wind_active'),group:2},
	                                            {type:'state',name:'wind',value:'2',title:_('wind inactive'),gridText:_('grid_wind_inactive'),group:2},
	                                            {type:'state',name:'hail',value:'1',title:_('hail active'),gridText:_('grid_hail_active'),group:3},
	                                            {type:'state',name:'hail',value:'2',title:_('hail inactive'),gridText:_('grid_hail_inactive'),group:3}]},
	                                   {type:'multiItem',title:_('Security'),entry: [
	                                           {type:'state',name:'fire',value:'1',title:_('fire active'),gridText:_('grid_fire_active'),group:1},
	                                           {type:'state',name:'fire',value:'2',title:_('fire inactive'),gridText:_('grid_fire_inactive'),group:1},
	                                            {type:'state',name:'panic',value:'1',title:_('panic_active'),gridText:_('grid_panic_active'),group:2},
	                                            {type:'state',name:'panic',value:'2',title:_('panic_inactive'),gridText:_('grid_panic_inactive'),group:2},
	                                            {type:'state',name:'alarm',value:'1',title:_('alarm_active'),gridText:_('grid_alarm_active'),group:3},
	                                            {type:'state',name:'alarm',value:'2',title:_('alarm_inactive'),gridText:_('grid_alarm_inactive'),group:3},
	                                            {type:'state',name:'alarm2',value:'1',title:_('alarm2_active'),gridText:_('grid_alarm2_active'),group:4},
	                                            {type:'state',name:'alarm2',value:'2',title:_('alarm2_inactive'),gridText:_('grid_alarm2_inactive'),group:4},
	                                            {type:'state',name:'alarm3',value:'1',title:_('alarm3_active'),gridText:_('grid_alarm3_active'),group:5},
	                                            {type:'state',name:'alarm3',value:'2',title:_('alarm3_inactive'),gridText:_('grid_alarm3_inactive'),group:5},
	                                            {type:'state',name:'alarm4',value:'1',title:_('alarm4_active'),gridText:_('grid_alarm4_active'),group:6},
	                                            {type:'state',name:'alarm4',value:'2',title:_('alarm4_inactive'),gridText:_('grid_alarm4_inactive'),group:6}]},
                                        {type:'zonestate',title:_('ZoneState')}
                            ],
                            flex : true
                        }]
                    }]
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.refetchDevices(grid);
                    },
                    'saveEntry':function (grid,values){    
                        
                        me.waitAndReload(grid,{
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        });
                    },
                    'enableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'enable',
                            id : Ext.JSON.encode(idVal)
                        });
                     },
                    'disableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'disable',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'deleteEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'testEntry':function (grid,entry){
                        dss.raiseEvent('event-push-notification', {
                            actions : 'test',
                            id : entry.get('id')
                        });
                    }
                }
        };
        
        if (getParameterByName('mode')=='expert') {
        	oConfig.grouperField='grouping';
        	oConfig.editorConfig.tabControls[0].content[0].userStates=true;
        }
        
        grid=Ext.create('DSS.gridPanel', oConfig);
        return grid;
    },
    loadDevices: function() {
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/event-push-notification/devices/*(name,uuid,appName)'
        });
        var result = Ext.JSON.decode(sString);
        if (result.result.devices==null) {
        	Ext.MessageBox.alert(_('No target devices are known'),_('No target devices are known. Please turn on the my.digitalstrom app and register some smartphones.'));
        	return;
        }
        deviceArray=result.result.devices;
        this.mobileSelector.updateCheckbox();
    },
    refetchDevices: function(grid) {
        var waitWindow = createWaitWindow();
    	grid.setStatus(_('Fetching devicelist from cloud ...'));
    	waitWindow.showWindow(_('Fetching devicelist from cloud ...'), true);
    	
    	
    	dss.pushAndQuery('event-push-notification',{actions : 'fetchDeviceList'},
		         'event-push-notification.saved',function(eventData) {
        	grid.setStatus(_('Refreshing device list'));
            me.loadDevices();
            me.loadParameter(grid);
            waitWindow.closeWindow();
            grid.setStatus(_('Ready'));

			 return true;
		});
    },
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
        var me = this;
        
    	dss.pushAndQuery('event-push-notification',parameter,
		         'event-push-notification.saved',function(eventData) {
   		 me.loadParameter(grid);
			 return true;
		});
        
        
      
    },    
    loadParameter : function(grid) {    	    	
    	grid.setStatus(_('Loading ...'));
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/event-push-notification/values/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                           {name:'id',type:'simple'},
                                                           {name:'disabled',type:'condition/enabled'},
                                                           {name:'message', type:'simple'},                                                           
                                                           {name:'triggers',type:'trigger'},
                                                           {name:'conditions', type: 'condition'}]);    
        if (result.result.values!=null) 
        	if (oResultObject!=null){
        	
        	for (var iIndex=0;iIndex<result.result.values.length;iIndex++) {
        		var simpleObject=result.result.values[iIndex];
        		var id=simpleObject.id;
        		var oUUID=[];
        		for (var sKey in simpleObject)
        			if (typeof (simpleObject[sKey]) == "object") {
        				for (var jIndex=0;jIndex<simpleObject[sKey].length;jIndex++)
        					if (simpleObject[sKey][jIndex].uuid!=null) {
        						var oUUIDContainer=simpleObject[sKey][jIndex];
        						for (var ssKey in oUUIDContainer) {
        							if (ssKey!='uuid') {
        								oUUID.push(oUUIDContainer[ssKey]);
        							}
        								
        						}
        					}
        			}
        		for (var jIndex=0;jIndex<oResultObject.length;jIndex++) {
        			if (oResultObject[jIndex].id==id)
        				oResultObject[jIndex].uuid=oUUID;
        		}
        	}
        	
        	
            for (var iIndex=0;iIndex<oResultObject.length;iIndex++) {
                oResultObject[iIndex].grouping=unescape(oResultObject[iIndex].name).split(' ')[0];
            }
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    }
    
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/event-push-notification.po']);
    var eventPushNotification = Ext.create('DSS.addon.EventPushNotification');
    eventPushNotification.initPage();
});
